/*
 * @(#)OldStyleAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.oldaglet;

import aglet.*;
import aglet.a4compatibility.*;

import ibm.aglets.util.SeqItinerary;

import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;
import java.net.URL;

/**
 * The <tt>SimpleAglet</tt> class is an example to show
 * how to migrate alpha4 to alpha5 style aglet. Here is how to
 * update your code to alpha5.
 * <ol>
 * <li> add the line in the import list.
 * <pre>
 * import aglet.a4compatibility.*;
 * </pre>
 * <li> extends OldAglet instead of Aglet like
 *  <pre>
 *  public class MyAglet extends OldAglet {
 *  }
 *  </pre>
 * <li> modify the "onCreation" method with a call to the "onCreation" 
 *      method on the super class (as the last statement), as follows:
 * <pre>
 *    public void onCreation(Object init) {
 *        .... // your initialization code goes here
 *
 *        super.onCreation(init);
 *    }
 * </pre>
 * <li> call writeExternal/readExternal method on the super class if the
 *      aglet implements Externalizable interface.
 * <pre>
 *    public void writeExternal(ObjectOutput out) throws IOException {
 *        super.writeExternal(out);
 *        // your code goes here....
 *    }
 *    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
 *        super.readExternal(in);
 *        // your code goes here....
 *    }
 * </pre>
 * </ol>
 *
 * <h3> What's not compatible? </h3>
 * Although this package provides high compatibility, there are a couple of
 * incompatibility which is not supoprted by this package.
 * <ol>
 * <li> Exceptions thrown by the method has been changed. You have to
 *     modify your code in order to conform to the new API. Most of
 *     changes are elimination of <tt> InvalidAgletException </tt>from methods
 *     on the <tt> Aglet </tt> class, so it would not be so difficult.
 * <li> The property has been removed from J-AAPI. The property releated
 *      method defined on <tt> AgletProxy</tt> in the previous release(alpha4)
 *      are not supported in this compatibility package.
 *     (e.g. AgletProxy.getProperty(String)  )
 * </ol>
 *
 * @version     1.00    $Date: 7/08/97 2:07p $
 * @author      Danny B. Lange
 * @author      Mitsuru Oshima
 * @see aglet.a4compatibility.OldAglet
 * @see aglet.a4compatibility.Itinerary
 * @see aglet.a4compatibility.AgletIdentifier
 */
public class OldStyleAglet extends OldAglet implements Externalizable {
    SeqItinerary itinerary;
    boolean done = false;

    public void onCreation(Object o) {
	print("Creation.");

	try {
	    URL dest = new URL("atp://sirius.trl.ibm.com:2000");
	    itinerary = new SeqItinerary(dest);
	    setItinerary(itinerary);
	} catch (AgletException ex) {
	    ex.printStackTrace();
	} catch (IOException ex) {
	    ex.printStackTrace();
	}

	//
	// These methods, setProperty() and getIdentifier(), are
	// defined in OldAglet class.
	//
	try {
	    setProperty("myId", getIdentifier().toString() );
	} catch (InvalidAgletException ex) {
	    ex.printStackTrace();
	}

	super.onCreation(o);
    }

    public void onClone() {
	print("Clone.");
    }

    public void onArrival() {
	print("Arrival.");
    }

    public void onActivation() {
	print("Activation.");
    }

    public void onCloning() {
	print("Cloning.");
    }

    public void onDispatching(URL url) {
   	print("Dispatching to " + url.toString() + ".");
    }

    public void onDisposing() {
	print("Disposing.");
    }

    public void onDeactivating(long time) {
	print("Deactivating.");
    }

    public void onReverting(URL remoteURL) {
	print("Retracted by " + String.valueOf(remoteURL) + ".");
    }

    public void run() {
	print("Run");
	if (itinerary.isAtLastDestination()) {
	    try {
		String id = getProperty("myId");
		setText( "Hello! my id is " + id);
		done = true;
		getMessageManager().waitMessage(2*1000); // 2 second
		itinerary.gotoOrigin();
	    } catch (AgletException ex) {
		ex.printStackTrace();
	    } catch (IOException ex) {
		ex.printStackTrace();
	    }
	} else if (itinerary.isAtOrigin() && done) {

	    //
	    // You need to remove the try/catch block because
	    // dispose() method no longer throws a InvalidAgletException
	    //

//	    try {
		dispose();
//	    } catch (InvalidAgletException ex) {
//	    }

	} else {
	    try {
		itinerary.gotoNextAvailable();
	    } catch (AgletException ex) {
		ex.printStackTrace();
	    } catch (IOException ex) {
		ex.printStackTrace();
	    }
	}
    }

    public boolean handleMessage(Message msg) {
	print("Receive " + msg );
	return true;
    }

    private void print(String m) {
   	System.out.println("SimpleAglet: " + m);
    }

    public void writeExternal(ObjectOutput out) throws IOException {
	//
	// If you're using Externalizable interface to serialize your
	// aglet, please remind that you have to call
	//    super.writeExternal(out);
	// method on the super class to serialize whole state.
	//
	super.writeExternal(out);

	out.writeObject(itinerary);
	out.writeBoolean(done);
	print("WriteExternal");
    }

    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
	//
	// If you're using Externalizable interface to deserialize your
	// aglet, please remind that you have to call
	//    super.readExternal(in);
	// method on the super class to deserialize whole state.
	//
	super.readExternal(in);

	itinerary = (SeqItinerary)in.readObject();
	done = in.readBoolean();
	print("ReadExternal");
    }
}

