/*
 * @(#)AgletContext.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet;

import aglet.system.ContextEvent;
import aglet.system.ContextListener;

import java.util.Properties;
import java.util.Enumeration;
import java.io.IOException;
import java.net.URL;

import java.awt.Image;
import java.applet.AudioClip;

/**
 * The <tt>AgletContext</tt> interface is used by an aglet to get information
 * about its environment and to send messages to the environment and other
 * aglets currently active in that environment. It provides means for
 * maintaining and managing running aglets in an environment where the host
 * system is secured against malicious aglets. 
 *
 * @version     1.50	$Date: 7/08/97 3:31p $
 * @author      Danny B. Lange
 * @author      Mitsuru Oshima
 */
public interface AgletContext {

    /**
     * Gets the name of the context
     * @return a name of aglet context
     */
    abstract public String getName();

    /**
     * Gets an enumeration of all aglets in the current context including
     * deactivated aglets. This will be changed so that it returns an
     * array of proxies.
     * @return a list of proxies.
     * @deprecated
     */
    abstract public Enumeration getAgletProxies();

    /**
     * Gets an array of aglet proxies in the current context. 
     * @see Aglet#ACTIVE
     * @see Aglet#INACTIVE
     * @param type the type of aglets.  ACTIVE, INACTIVE or ACTIVE | INACTIVE,
     * @return a list of proxies.
     */
    abstract public AgletProxy[] getAgletProxies(int type);

    /**
     * Gets a proxy for an aglet in the current context. The selected aglet
     * is specified by its identity.
     * @param id the identity of the aglet.
     * @return the proxy.
     */
    abstract public AgletProxy getAgletProxy(AgletID id);

    /**
     * Gets a proxy for an aglet in a remote context. The remote context is
     * identified by its URL, and the aglet is indicated by its identifier.
     *
     * @param identity the identity of the aglet.
     * @return the proxy.
     * @deprecated
     */
    abstract public AgletProxy getAgletProxy(URL host, AgletID id);

    /**
     * Creates an instance of the specified aglet class. The aglet's class
     * code file can be located on the local file system as well as on a
     * remote server. If the <tt>codeBase</tt> is <tt>null</tt>, the context
     * will search for the code in the local system's aglet search path
     * (<tt>AGLET_PATH</tt>). The createAglet method takes three arguments:
     * <tt>codeBase</tt>, <tt>code</tt>,  and <tt>init</tt>:<p>
     * <ol>
     * <li><tt>codeBase</tt> specifies the base URL of the aglet class file,
     * in other words, the (possibly remote) directory that contains the
     * aglet's code. If this argument is <tt>null</tt>, then the directories
     * specified in the local host's aglet search path are searched.
     * The aglet search path works in a similar way to Java's class path. 
     * It is typically an environment variable that specifies a list of
     * directories to be searched for aglet code files.</li>
     * <li><tt>code</tt> gives the name of the file that contains the
     * aglet's compiled class  code. This file is relative to the base URL
     * of the aglet, and cannot be absolute.</li> 
     * <li><tt>init</tt> is an object passed on to the aglet's onCreation
     * method. </li>
     * </ol>
     * @param codeBase base URL of the aglet class file.
     * @param code name of the aglet's compiled class file.
     * @param init initialization argument.
     * @return the proxy of the new aglet.
     * @exception UnknownHostException if the given host could not be found.
     * @exception ServerNotFoundException if the server could not be found.
     * @exception InstantiationException if the instantiation failed.
     * @exception ClassNotFoundException if the class not found.
     * @see Aglet#onCreation
     */
    abstract public AgletProxy createAglet(URL codeBase, String code, Object init)  throws IOException, AgletException, ClassNotFoundException, InstantiationException;

    /**
     * Retracts the aglet specified by its url:
     * <tt>atp://host-domain-name/#aglet-identity</tt>.
     * @param url the location and aglet identity of the aglet to be retracted.
     * @return the aglet proxy for the retracted aglet.
     * @exception UnknownHostException if the specified HOST is not found.
     * @exception ServerNotFoundException if the aglet server specified
     *            in the URL is not available.
     * @exception MalformedURLException if the given url is not URI for
     *            an aglet.
     * @exception RequestRefusedException if the retraction refused.
     * @exception AgletNotFoundException if the aglet could not be found.
     *
     * @deprecated
     */
    abstract public AgletProxy retractAglet(URL url) throws IOException, AgletException;

    /**
     * Retracts the aglet specified by its url and id
     * @param url the location of the aglet to be retracted.
     * @param id  the aglet identity of the aglet to be retracted.
     * @return the aglet proxy for the retracted aglet.
     * @exception UnknownHostException if the specified HOST is not found.
     * @exception ServerNotFoundException if the aglet server specified
     *            in the URL is not available.
     * @exception MalformedURLException if the given url is not URI for
     *            an aglet.
     * @exception RequestRefusedException if the retraction refused.
     * @exception AgletNotFoundException if the aglet could not be found.
     */
    abstract public AgletProxy retractAglet(URL url, AgletID aid) throws IOException, AgletException;

    /**
     * Activates an aglet. This is a forced activation of a deactivated aglet.
     * @param aid aglet identifier.
     * @return proxy of the activated aglet.
     * @exception AgletNotFoundException if the aglet could not be found.
     *
     * @deprecated
     * This will be removed from API in the future release.
     * replaced with AgletProxy.activate();
     */
    abstract public AgletProxy activateAglet(AgletID aid) throws IOException, AgletException;

    /**
     * Sends a multicast message to the subscribers in the context
     * @param message to send
     * @return ReplySet containing FutureReplies
     */
    abstract public ReplySet multicastMessage(Message msg);

    /**
     * Returns the URL of the daemon serving this context.
     * @return the URL of the daemon. <tt>null</tt> if the hosting information is not available.
     */
    abstract public URL getHostingURL();

    /**
     * Shows a new document. This may be ignored by the aglet context.
     * @param url an url to be shown
     */
    abstract public void showDocument(URL url);

    /**
     * Gets the context property indicated by the key.
     * @param key the name of the context property.
     * @return the value of the specified key.
     */
    abstract public Object getProperty(String key);

    /**
     * Gets the context property indicated by the key and default value.
     * @param key the name of the context property.
     * @param def the value to use if this property is not set.
     * @return the value of the specified key.
     */
    abstract public Object getProperty(String key, Object def);

    /**
     * Sets the context property indicated by the key and value.
     * @param key the name of the context property.
     * @param value the value to be stored.
     * @return the value of the specified key.
     */
    abstract public void setProperty(String key, Object value);

    /*
     * Media API
     */

    /**
     * Gets an image
     */
    abstract public Image getImage(URL image);

    /**
     * Gets an audio clip
     */
    abstract public AudioClip getAudioClip(URL audio);

    /**
     * Gets an image data. This is a tempolary solution.
     */
    abstract public ImageData getImageData(URL image);

    /**
     * Gets an image. This is a tempolary solution.
     */
    abstract public Image getImage(ImageData image);

    /**
     * Adds the specified context listener to receive context events
     * from this context.
     * @param listener the context listener
     * @exception SecurityException
     */
    abstract public void addContextListener(ContextListener listener);

    /**
     * Removes the specified context listener.
     * @param listener the context listener
     * @exception SecurityException
     */
    abstract public void removeContextListener(ContextListener listener);

    /**
     * Starts the context. This is ignored if the context is already runnig.
     * @exception SecurityException
     */
    abstract public void start();

    /**
     * Shutdown the context with the specific message object. This messag
     * object is delivered to all aglets in the context before all aglets
     * are killed. This is ignored if the context is already stopped.
     * @exception SecurityException if the current execution context is not
     * allowd to shutdown
     * @see start
     */
    abstract public void shutdown(Message msg);

    /**
     * Shutdown the context. This is ignored if the context is already stopped.
     * @exception SecurityException if the current execution context is not
     * allowd to shutdown
     * @see start
     */
    abstract public void shutdown();
}
