/*
 * @(#)AgletProxy.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet;

import aglet.a4compatibility.AgletIdentifier;

import java.net.URL;
import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;

/**
 * AgletProxy interface is a placeholder for aglets. The purpose of this
 * interface is to provide a mechanism to control and limit direct access to
 * aglets.
 *
 * @version     1.50    $Date: 7/07/97 6:06p $
 * @author      Danny B. Lange
 * @author	Mitsuru Oshima
 */
public interface AgletProxy {

    /**
     * Gets the aglet that the proxy manages.
     * @return the aglet
     * @exception InvalidAgletException if the aglet is not valid.
     * @exception SecurityException if you are not allowed to access the aglet.
     */
    abstract public Aglet getAglet() throws InvalidAgletException;

    /**
     * Gets the aglet's identifier.
     * @return the aglet's identifier.
     * @exception InvalidAgletException if the aglet is not valid.
     * @deprecated
     */
    abstract public AgletIdentifier getIdentifier() throws InvalidAgletException;
    /**
     * Gets the aglet's id
     * @return the aglet's id
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract public AgletID getAgletID() throws InvalidAgletException;

    /**
     * Gets the aglet's class name.
     * @return the class name.
     * @exception InvalidAgletException if the aglet is not valid.
     * @deprecated
     */
    abstract public String getAgletClassName() throws InvalidAgletException;

    /*
     * Gets the information object of the aglet
     *
     * @return the AgletInfo object
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract public AgletInfo getAgletInfo() throws InvalidAgletException;

    /**
     * Gets the codebase of the aglet in URL
     * @return the class URL.
     * @exception InvalidAgletException if the aglet is not valid.
     *
     * @deprecated as of Aglets1.0a5
     * use AgletInfo.getCodeBase instead
     */
    abstract public URL getCodeBase() throws InvalidAgletException;

    /*
     * Aglet Control APIs.
     */

    /**
     * Clones the aglet and its proxy. 
     * Note that the cloned aglet will get activated.
     * If you like to get cloned aglet which is not activated, throw
     * ThreadDeath exception in the onClone method.
     *
     * @return  the new aglet proxy what holds cloned aglet.
     * @exception CloneNotSupportedException if the cloning fails.
     * @exception InvalidAgletException if the aglet is invalid.
     */
    abstract public Object clone() throws CloneNotSupportedException;

    /*
     * Dispatches the aglet to the location specified by the argument url.
     * @param url dispatch destination.
     * @return the AgletProxy of the dispatched (remote) aglet.
     * @exception ServerNotFoundException if the server
     * @exception UnknownHostException if the host given in the URL doesn't
     *            exist.
     * @exception MalformedURLException if the given url dosn't specify the
     *            host.
     * @exception RequestRefusedException if the remote server refused the
     *            dispatch request.
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract public AgletProxy dispatch(URL url) throws IOException, AgletException;

    /**
     * Disposes the aglet.
     * @exception InvalidAgletException if the aglet is invalid.
     */
    abstract public void dispose() throws InvalidAgletException;


    /**
     * Deactivates the aglet. The system may store the aglet in the spool
     * (disk or memory depending on the server). The aglet will be re-activated
     * later (at the given time or manually).
     * @param millisecounds duration of the aglet deactivating.
     * @exception InvalidAgletException if the aglet is not valid.
     * @exception IllegalArgumentException if the minutes parameter is negative.
     */
    abstract public void deactivate(long millisecounds) throws IOException, InvalidAgletException;

    /**
     * Activates the aglet. This is a forced activation of a deactivated aglet.
     * @exception IOException if the activation failed
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract public void activate() throws IOException, AgletException;

    /**
     * Sends a message in synchronous way. This waits for finishing the message
     * handing.
     * @param msg a message to send
     * @return the result object if any. null if not.
     * @exception InvalidAgletException if the aglet is not valid any longer.
     * @exception NotHandledException if the aglet didn't handle the
     *            the message.
     * @exception MessageException a exception 
     *            which the handleMessage method raised.
     */
    abstract public Object sendMessage(Message msg) throws InvalidAgletException, NotHandledException, MessageException;

    /**
     * Sends a message in asynchronous way.
     * @param msg a message to send
     * @return a future object that will give you the reply of the message.
     * @exception InvalidAgletException if the aglet is not valid any longer.
     */
    abstract public FutureReply sendAsyncMessage(Message msg) throws InvalidAgletException;

    /**
     * Sends a future message to the aglet. The invocation will
     *
     * @param msg a message to send
     * @return a future object that will give you the reply of the message.
     * @exception InvalidAgletException if the aglet is not valid any longer.
    abstract public FutureReply sendFutureMessage(Message msg) throws InvalidAgletException;
     */

    /**
     * Sends a oneway message to the aglet. No acknowledgement will be
     * sent back to the sender.
     * @param msg a message to send
     * @exception InvalidAgletException if the aglet is not valid any longer.
    abstract public void sendOnewayMessage(Message msg) throws InvalidAgletException;
     */

    /**
     * Delegates a message to the aglet
     * @param msg a message to delegate
     * @exception InvalidAgletException if the aglet proxy is not valid.
     */
    abstract public void delegateMessage(Message msg) throws InvalidAgletException;

    /**
     * Checks if the aglet is active or deactivated.
     * @return true if the aglet is active
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract public boolean isActive() throws InvalidAgletException;

    /**
     * Checks if the aglet proxy is invalidated or not.
     * @return true if the aglet proxy is valid. false if not.
     */
    abstract public boolean isValid();

    /**
     * Checks if the aglet proxy is referencing a remote aglet.
     * @return true if the aglet resides at the remote site
     */
    abstract public boolean isRemote();

    /**
     * Checks if the aglet
     * @see aglet.Aglet#ACTIVE
     * @see aglet.Aglet#INACTIVE
     */
    abstract public boolean isState(int type);
}
