/*
 * @(#)Message.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet;

import java.util.Hashtable;

/**
 * The <tt>Message</tt> class is a object that holds its kind 
 * and arguments passed to the receiver. In handleMessage() method
 * on Aglet class, the reply to the request can be set if any.
 *
 * @version     1.50    $Date: 7/08/97 11:33a $
 * @author	Mitsuru Oshima
 */
public class Message implements java.io.Serializable {
    /**
     * An arbitrary argument. Aglet programmers should NOT use this
     * member variable because this will be removed from public
     * interface in the future.
     * @deprecated
     */
    public Object arg;

    /**
     * The kind of the message. Aglet programmers should NOT use this
     * member variable because this will be removed from public
     * interface in the future.
     * @deprecated
     */
    public String kind;

    /**
     * The time when the message was sent. Aglet programmers should NOT
     * use this member variable because this will be removed from public
     * interface in the future.
     * @deprecated
     */
    public long timestamp;

    /**
     * These kinds are used to specify the priority of the system message
     * @see MessageManager#setPriority
     */
    static final public String CLONE      = "_clone";
    static final public String DISPATCH   = "_dispatch";
    static final public String DISPOSE    = "_dispose";
    static final public String DEACTIVATE = "_deactivate";
    static final public String REVERT     = "_revert";

    /**
     * Constructs a message. The message object created by
     * this constructor have a hashtable which can be used
     * for argument-value pair.
     *
     * <pre>
     *   Message msg = new Message("stock-price");
     *   msg.setArg("company", "ibm");
     *   msg.setArg("currency", "dallar");
     *   Double d = (Double) proxy.sendMessage(msg);
     * </pre>
     * 
     * @param kind a kind of this message
     */
    public Message(String kind) {
	this(kind, new Hashtable());
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     * @param arg an argument of this message
     */
    public Message(String kind, Object arg) {
	this.kind = kind;
	this.arg = arg;
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     */
    public Message(String kind, int i) {
	this(kind, new Integer(i));
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     */
    public Message(String kind, double d) {
	this(kind, new Double(d));
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     */
    public Message(String kind, float f) {
	this(kind, new Float(f));
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     */
    public Message(String kind, boolean b) {
	this(kind, new Boolean(b));
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     */
    public Message(String kind, char c) {
	this(kind, new Character(c));
    }

    /**
     * Constructs a message with an argument value.
     * @param kind a kind of this message
     */
    public Message(String kind, long l) {
	this(kind, new Long(l));
    }

    /**
     * Gets the kind of this message
     */
    public String getKind() {
	return kind;
    }

    /**
     * Checks if the message has same kind as given string.
     * @param k a string to compare
     */
    public boolean sameKind(String k) {
	return kind.equals(k);
    }

    /**
     * Gets the time in milliseconds when the message was sent.
     */
    public long getTimeStamp() {
	return timestamp;
    }

    /**
     * Gets the argument
     */
    public Object getArg() {
	return arg;
    }


    /**
     * Sets an argument value with associated name.
     * @param name a name of this argument
     * @param a a value of this argument
     */
    public void setArg(String name, Object a) {
	if (arg instanceof Hashtable == true) {
	    ((Hashtable)arg).put(name, a);
	} else {
	    throw new RuntimeException("Cannot set name-value pair");
	}
    }

    /**
     * Gets an argument value
     * @param name a name of this argument
     * @return a value of this argument
     */
    public Object getArg(String name) {
	if (arg instanceof Hashtable) {
	    return ((Hashtable)arg).get(name);
	}
	return null;
    }

    /**
     * Sets a reply to this message.
     * @see FutureReply#getReply
     * @exception IllegalAccessErrro if a reply has already been sent.
     */
    public void sendReply(Object arg) {
	throw new NoSuchMethodError();
    }

    /**
     * Send a reply without sepcific value.
     * @exception IllegalAccessErrro if a reply has already been sent.
     */
    public void sendReply() {
	throw new NoSuchMethodError();
    }

    /**
     * Sets a exception to this message.
     * @exception IllegalAccessErrro if a reply has already been sent.
     */
    public void sendException(Exception exp) {
	throw new NoSuchMethodError();
    }

    /**
     * Sends a integer value as a reply
     */
    public void sendReply(int i) {
	sendReply(new Integer(i));
    }

    /**
     * Sends a double value as a reply
     */
    public void sendReply(double d) {
	sendReply(new Double(d));
    }

    /**
     * Sends a float value as a reply
     */
    public void sendReply(float f) {
	sendReply(new Float(f));
    }

    /**
     * Sends a bolean value as a reply
     */
    public void sendReply(boolean b) {
	sendReply(new Boolean(b));
    }

    /**
     * Sends a character value as a reply
     */
    public void sendReply(char c) {
	sendReply(new Character(c));
    }

    /**
     * Sends a long value as a reply
     */
    public void sendReply(long l) {
	sendReply(new Long(l));
    }

    /**
     * Compares two Message objects. Use sameKind() method to compare
     * a message and a kind.
     * @see Message#sameKind
     */
    public boolean equals(Object obj) {
	if (obj instanceof Message) {
	    return ((Message)obj).sameKind(kind);
	}
	return false;
    }

    public String toString() {
	return "[kind = " + kind + ": arg = " + String.valueOf(arg) + ']';
    }
}
