/*
 * @(#)OldAglet.java
 *
 * (c) Copyright IBM Corp. 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet.a4compatibility;

import aglet.*;
import aglet.event.*;

import java.awt.Image;
import java.net.URL;
import java.util.Properties;
import java.util.Enumeration;
import java.io.IOException;

/**
 * OldAglet class provides a backward compatibility interface for the
 * alpha4 aglets. To update the aglets written in Alpha4 API to
 * Alpha5 API, follow the instruction below
 * <ol>
 * <li> Add the aglet.a4compatibility package to the import list
 *     <pre>
 *       package your.package;
 *       import aglet.a4compatibility.*;
 *     </pre>
 * <li> replace the name of superclass 'Aglet' with 'OldAglet'.
 * <li> If the aglet has the method 'onCreation' defined,
 *      add the statement <tt>super.onCreation(null)</tt> at the bottom
 *      of that <tt>onCreation</tt> method.
 *     <pre>
 *       public void onCreation(Object init) {
 *           // your code goes here
 *           super.onCreation(null);
 *       }
 *     </pre>
 * <li> The exceptions thrown by the method on Aglet class has been changed.
 *      Please made modification where the compiler complains about the
 *      exceptions.
 * </ol>
 *
 * @version	1.00	$Date: 7/08/97 3:31p $
 * @author	Mitsuru Oshima
 */
public abstract class OldAglet extends Aglet implements MobilityListener, CloneListener, PersistencyListener {

    public OldAglet() {
    }

    /**
     *
     */
    public void onCreation(Object ini) {
	addCloneListener(this);
	addMobilityListener(this);
	addPersistencyListener(this);

	MessageManager mm = getMessageManager();
	mm.setPriority(Message.CLONE,      MessageManager.NOT_QUEUED);
	mm.setPriority(Message.DISPOSE,    MessageManager.NOT_QUEUED);
	mm.setPriority(Message.DEACTIVATE, MessageManager.NOT_QUEUED);
	mm.setPriority(Message.DISPATCH,   MessageManager.NOT_QUEUED);
    }

    /**
     * Old Callbacks
     */
    public void onCloning() {
    }

    public void onClone() {
	throw new SecurityException("onClone");
    }

    public void onDispatching(URL url) {
    }

    public void onArrival() {
    }

    public void onReverting(URL remoteURL) {
    }

    public void onDeactivating(long duaration) {
    }

    public void onActivation() {
    }

    /*
     * Delegation Method
     */
    final public void onCloning(CloneEvent ev) {
	try {
	    onCloning();
	} finally {
	    unsetTraveller();
	}
    }

    final public void onClone(CloneEvent ev) {
	setTraveller();
	onClone();
    }

    final public void onCloned(CloneEvent ev) {
	setTraveller();
    }

    final public void onDispatching(MobilityEvent ev) {
	try {
	    onDispatching(ev.getLocation());
	} finally {
	    unsetTraveller();
	}
    }

    final public void onArrival(MobilityEvent ev) {
	setTraveller();
	onArrival();
    }

    final public void onReverting(MobilityEvent ev) {
	try {
	    onReverting(ev.getLocation());
	} finally {
	    unsetTraveller();
	}
    }

    final public void onDeactivating(PersistencyEvent ev) {
	try {
	    onDeactivating(ev.getDuration());
	} finally {
	    unsetTraveller();
	}
    }

    final public void onActivation(PersistencyEvent ev) {
	setTraveller();
	onActivation();
    }

    /**
     * Returns the identity of this aglet.
     * @exception InvalidAgletException if the aglet is not valid.
     * @deprecated
    public final AgletIdentifier getIdentifier() {
	return new AgletIdentifier(getAgletInfo().getAgletID());
    }
     */

    /**
     * Gets the aglet property indicated by the key.
     * @param key the name of the aglet property.
     * @return the value of the specified key.
     */
    public final String getProperty(String key) throws InvalidAgletException {
	return properties.getProperty(key);
    }

    /**
     * Gets the aglet property indicated by the key and default value.
     * @param key the name of the aglet property.
     * @param defValue the default value to use if this property is not set.
     * @return the value of the specified key.
     */
    public final String getProperty(String key, String defValue) throws InvalidAgletException {
	return properties.getProperty(key, defValue);
    }

    /**
     * Sets the aglet property indicated by the key and the value.
     * @param key the name of the aglet property.
     * @param value the value to put
     */
    public final void setProperty(String key, String value) throws InvalidAgletException {
	properties.put(key, value);
    }

    /**
     * Enumerates all the property keys.
     * @return property key enumeration.
     */
    public final Enumeration getPropertyKeys() throws InvalidAgletException {
	return properties.keys();
    }

    public Image getIconImage() {
	return null;
    }

    /**
     * Assigns an Itinerary object to the aglet. This assignment is 
     * allowed only once during the life-time of the aglet.
     * @param itinerary the Itinerary object.
     * @exception AgletException if the aglet was already assigned an 
     * itinerary.
     */
    public synchronized final void setItinerary(Itinerary i) throws AgletException {
	if (itinerary != null) {
	    itinerary.unsetTraveller();
	}
	itinerary = i;
	setTraveller();
	itinerary.initialize();
    }

    /**
     * @return the itinerary Object of the aglet; null otherwise.
     * @exception InvalidAgletException if the aglet is not valid.
     */
    public final Itinerary getItinerary() throws InvalidAgletException {
	return itinerary;
    }

    /*
     *
     */
    private void unsetTraveller() {
	if (itinerary != null) {
	    itinerary.unsetTraveller();
	}
    }

    /*
     *
     */
    private void setTraveller() {
	AgletProxy proxy = getAgletContext().getAgletProxy(getIdentifier());
	if (itinerary != null) {
	    itinerary.setTraveller(proxy);
	}
    }

    /**
     * Writes a state of this aglet to the ObjectOutput. this is used 
     * by a subclass which implemetns Externalizable
     */
    protected void writeExternal(java.io.ObjectOutput s) throws IOException {
	s.writeObject(itinerary);
	s.writeObject(properties);
    }

    /**
     * Reads a state of this aglet from the ObjectInput. this is used 
     * by a subclass which implemetns Externalizable
     */
    protected void readExternal(java.io.ObjectInput s) throws IOException, ClassNotFoundException {
	itinerary = (Itinerary)s.readObject();
	properties = (Properties)s.readObject();
    }

    private Itinerary itinerary = null;
    private Properties properties = new Properties();
}
