/*
 * @(#)AgletRuntime.java
 *
 * (c) Copyright IBM Corp. 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet.system;

import aglet.Message;
import aglet.AgletContext;
import aglet.Aglet;
import aglet.AgletID;
import aglet.AgletProxy;
import java.net.URL;

/**
 * <tt>AgletRuntime</tt> class provides the way to access the information
 * of the local or remote context. Neither of an application nor aglet
 * can create its own instance of this runtime class.
 *
 * @version     1.00    $Date: 7/09/97 1:20a $
 * @author      Mitsuru Oshima
 * @see AgletRuntime#getAgletRuntime
 */
public abstract class AgletRuntime {

    //
    // These is a quick hack to work around the problem in Fiji
    //
    static private String tmp;   

    static {
	try {
	   tmp=System.getProperty("aglet.runtime.packagePrefix", "ibm.aglets");        } catch (SecurityException e) {  // by Fiji
	    e.printStackTrace();
	    tmp="ibm.aglets";
	}
    }

    final static String runtimePackagePrefix = tmp;
//	    System.getProperty("aglet.runtime.packagePrefix", "ibm.aglets");

    /**
     * Gets the AgletRuntime object associated with the current
     * Java application.
     */
    synchronized static public AgletRuntime getAgletRuntime() {
	if (runtime == null) {
	    String classname = runtimePackagePrefix + ".AgletRuntime";
	    try {
		Class clazz = Class.forName( classname );
		Object obj = clazz.newInstance();
		if (obj instanceof AgletRuntime) {
		    runtime = (AgletRuntime)obj;
		} else {
		    System.out.println("[ \""+ classname + "\" is not Runtime]");
		}
	    } catch (ClassNotFoundException ex) {
		System.out.println("[ The class \""+ classname + "\" not found]");
	    } catch (Exception ex) {
		System.out.println("[ An instance of \""+ classname +
				   "\" cannot be created]");
	    }
	}
	return runtime;
    }

    /**
     * Creates an DefaultAgletContext object given by the Framework
     * implementation with specified name. Hosting multiple contexts
     * is not supported in alpha5 release.
     * @exception SecurityException if the current execution is not allowed
     *  to create an AgletContext.
     */
    abstract public AgletContext createAgletContext(String name);

    /**
     * Returns the AgletContext which has the specified name.
     *
     * @param name the name of the context
     * @exception SecurityException if the current execution is not allowed
     *  to acccess the AgletContext.
     */
    abstract public AgletContext getAgletContext(String name);

    /**
     * Removes the specified aglet context from the runtime environment.
     * It is also removed from export list if it's exported.
     * @param cxt the context to remove
     */
    abstract public void removeAgletContext(AgletContext cxt);

    /**
     * Gets the contexts in the environment.
     */
    abstract public AgletContext[] getAgletContexts();

    /**
     * Creates an aglet remotely within the specified context.
     *
     * @param contextAddress an address of context.
     * @param codebase an codebase for the aglet.
     * @param name name of aglets' class.
     * @param init an object passed as an initialize argument.
     * @see aglet.Aglet#onCreation
     * @see aglet.AgletContext#createAglet
     */
    abstract public AgletProxy createAglet(String contextAddress,
					   URL codebase,
					   String name,
					   Object init);

    /**
     * Obtains the remote proxy for the aglet specified by the context and
     * id.
     * @param contextAddress
     * @param id
     */
    abstract public AgletProxy getAgletProxy(String contextAddress,AgletID id);

    /**
     * Returns the proxies in the context specified by the contextAddress.
     * @param contextAddress
     */
    abstract public AgletProxy[] getAgletProxies(String contextAddress);

    /**
     * Returns an enumeration of proxies in the context specified by
     * the contextAddress.
     * @param contextAddress
    abstract public Enumeration getAgletProxies(String contextAddress, String authorityName);
     */

    /**
     * Gets an address of the server in URL.
     * @return an URL represents the address of the server
     */
    abstract public URL getServerURL();

    /**
     * Shutdown all contexts in the runtime
     */
    abstract public void shutdown();

    /**
     * Shutdown all contexts in the current runtime with the specific message
     * object. This messag object is delivered to all aglets in all contexts
     * before all aglets are killed.
     */
    abstract public void shutdown(Message msg);

    /**
     * [Preliminary]
     * Exports the specified aglet context. After this successfull
     * invocation, the specified context is visiable and accessible
     * from a remote host, which means that the aglet can be dispatched
     * into and also the remote operations like listing the aglet proxies
     * residing in the context are enabled.
     * @param cxt the context to export
     * @exception SecurityException if the current execution is not allowed
     *  to export an AgletContext.
    abstract public void exportAgletContext(AgletContext cxt);
     */

    /**
     * [Preliminary]
     * Exports the specified aglet. After this successfull
     * invocation, the specified aglet is visiable and accessible
     * from a remote host, which means that the aglet can be receive
     * remote messages from remote hosts.
     * @param aglet the aglet to export
     * @exception SecurityException if the current execution is not allowed
     *  to export an Aglet
    abstract public void exportAglet(Aglet aglet);
     */

    /*
    abstract public boolean isGuiAvailable();
    */
    private static AgletRuntime runtime = null;
}
