/*
 * @(#)SeqPlanItinerary.java
 *
 * (c) Copyright IBM Corp. 1996, 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet.util;

import aglet.*;
import aglet.event.*;

import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;

/**
 * <tt> SeqPlanItinerary </tt> defines the trip plan which has a sequence of
 * place-message pair. For example, the plan defined like:
 * <pre>
 *   itinerary.addPlan("atp://first", "job1");
 *   itinerary.addPlan("atp://second", "job2");
 * </pre>
 * first dispatches an aglet to the <tt> "atp://first" </tt> and
 * sends <tt> new Message("job1") </tt> message to the aglet. After
 * that message handling is completed, the itinerary dispatches the aglet
 * to the next address, <tt> "atp://second" </tt>, and send the
 * corresponding message <tt> new Message("job2", init) to the aglet again.
 * The order of plan is defined in the order <tt>addPlan</tt> method is called,
 * or you can insert a new plan item at the specified index by calling <tt>
 * insertPlanAt </tt>. To automatically dispatches an aglet to the next
 * address when the job is completed, the aglet have to have the following
 * block
 * <pre>
 *    boolean handleMessage(Message msg) {
 *        if (itinerary.handleMessage(msg)) {
 *            return true;
 *        }
 *        ...
 *    }
 * </pre>
 * in the handleMessage() method. Here is a full example which make use
 * of the <tt> SeqPlanItinerary </tt> class.
 *
 * <pre>
 *   SeqPlanItinerary itinerary;
 *   public void onCreation(Object init) {
 *   	itinerary = new SeqPlanItinerary(this);
 *      itinerary.addPlan("atp://first",  "job1");
 *      itinerary.addPlan("atp://second", new Message("job2", init));
 *      itinerary.addPlan("atp://thrid",  "job1");
 *      itinerary.addPlan("atp://fourth", "end");
 *      itinerary.startTrip();
 *   }
 *
 *   public boolean handleMessage(Message msg) {
 *       if (itinerary.handleMessage(msg)) {
 *	     return true; // itinerary has handled the message
 *       } else if (msg.sameKind("job1")) {
 *           // do job1.
 *           // This will be called at "atp://first" and "atp://thrid".
 *       } else if (msg.sameKind("job2")) {
 *           // do job2. This will be called at "atp://second".
 *           Object obj = msg.getArg(); // returns init value given in the
 *                                      // onCreation method.
 *       } else if (msg.sameKind("end")) {
 *           printResult();
 *           dispose();
 *       } else return false;
 *       return true;
 *   }
 * </pre>
 *
 * @version     1.00    $Date: 7/08/97 10:08a $
 * @author      Mitsuru Oshima
 * @see aglet.Aglet#handleMessage
 */
public class SeqPlanItinerary extends MobilityAdapter implements java.io.Serializable {
    private final static Message goNext = new Message("goNext", null);

    /**
     * Constructs a SeqPlanItinerary object with the specified owner aglet.
     * @param aglet the owner aglet
     */
    public SeqPlanItinerary(Aglet aglet) {
	this.aglet = aglet;
	aglet.addMobilityListener(this);
    }

    /**
     * Conditionally enables a repetition. If true, the index of plan
     * is set to 0 when it reaches to the end of plan.
     * @param b if true, repeats the plan; doesn't otherwise.
     */
    public void setRepeat(boolean b) {
	repeat = b;
    }

    /**
     * Checks if the itinerary is repeatition plan.
     */
    public boolean isRepeat() {
	return repeat;
    }

    /**
     * Returns the size of this plan
     */
    public int size() {
	return hosts.size();
    }

    /**
     * Returns the address at the specified index.
     */
    public String getAddressAt(int index) {
	return (String) hosts.elementAt(index);
    }

    /**
     * Returns the message at the specified index.
     */
    public Message getMessageAt(int index) {
	return (Message) messages.elementAt(index);
    }

    /**
     * Adds the new plan item specified by a pair of a address and a message.
     * This is added at the end of plan.
     * @param address  the address to go
     * @param msg      the message to be sent to the owner aglet
     */
    synchronized public void addPlan(String address, Message msg) {
	hosts.addElement(address);
	messages.addElement(msg);
    }

    /**
     * Adds the new plan item specified by a pair of a address and a message.
     * This is added at the end of plan.
     * @param address  the address to go
     * @param msg      the message to be sent to the owner aglet
     */
    public void addPlan(String address, String msg) {
	addPlan(address, new Message(msg));
    }

    /**
     * Inserts the new plan item specified by a pair of a address and
     * a message at the specified index.
     * @param address  the address to go
     * @param msg      the message to be sent to the owner aglet
     * @param index    the index
     */
    synchronized public void insertPlanAt(String address, Message msg, int index) {
	hosts.insertElementAt(address, index);
	messages.insertElementAt(msg, index);
    }

    /**
     * Inserts the new plan item specified by a pair of a address and
     * a message at the specified index.
     * @param address  the address to go
     * @param msg      the message to be sent to the owner aglet
     * @param index    the index
     */
    public void insertPlanAt(String address, String msg, int index) {
	insertPlanAt(address, new Message(msg), index);
    }

    /**
     * Removes the plan specified by the index.
     * @param index the index to remove.
     */
    synchronized public void removePlanAt(int index) {
	hosts.removeElementAt(index);
	messages.removeElementAt(index);
    }

    /**
     * Clears the whole plan.
     */
    synchronized public void clearPlan() {
	hosts = new Vector();
	messages = new Vector();
    }

    /**
     * Handles the message. SeqPlanItinerary object only handles
     * "goToNext" message.
     * @return dispatches the aglet if the message is "goToNext",
     *         return false otherwise.
     */
    public boolean handleMessage(Message msg) {
	if (msg.equals(goNext)) {
	    return goToNext();
	}
	return false;
    }

    /**
     * Go to the next address and perform the next message at the
     * destination address.
     */
    public boolean goToNext() {
	try {
	    if (index >= hosts.size()) {
		return false;
	    }
	    URL address = new URL( (String) hosts.elementAt(index) );
	    aglet.dispatch( address );
	} catch (AgletException ex) {
	    // try next one
	    return tryNext();
	} catch (java.io.IOException ex) {
	    return tryNext();
	}
	return false;
    }

    /**
     * Starts the trip defined in this itineray
     */
    public void startTrip() throws java.io.IOException, AgletException {
	index = 0;
	goToNext();
    }

    private boolean tryNext() {
	// try next one
	index++;
	if (index >= hosts.size() && repeat) {
	    index = 0;
	}
	return goToNext();
    }

    /**
     * This is not normally used by aglets programmers.
     */
    public void onArrival(MobilityEvent ev) {
	try {
	    Message msg = (Message)messages.elementAt(index);
	    index ++;

	    if (index >= messages.size() && repeat) {
		index = 0;
	    }

	    if (msg != null) {
		ev.getAgletProxy().sendAsyncMessage(msg);
	    }

	    ev.getAgletProxy().sendAsyncMessage(goNext);

	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    private Aglet aglet;
    private Vector hosts = new Vector();
    private Vector messages = new Vector();
    private int index = 0;
    private boolean repeat = false;
}
