/*
 * @(#)SimpleItinerary.java
 *
 * (c) Copyright IBM Corp. 1996, 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet.util;

import aglet.*;
import aglet.event.*;

import java.util.Hashtable;
import java.util.Enumeration;
import java.net.URL;

/**
 * SimpleItinerary class is an itinerary object which can specify a
 * destination and a message which will be sent to the owner aglet
 * when it arrived at the destination.
 *
 * <pre>
 *   SimpleItinerary itinerary;
 *   public void onCreation(Object init) {
 *   	itinerary = new SimpleItinerary(this);
 *      itinerary.go("atp://first", "job1");
 *   }
 *
 *   public boolean handleMessage(Message msg) {
 *       if (msg.sameKind("job1")) {
 *           // job at the first place
 *           itinerary.go("atp://second", "job2");
 *       } else if (msg.sameKind("job2")) {
 *           // job at the second place
 *           itinerary.go("atp://third", "job3");
 *       } else if (msg.sameKind("job3")) {
 *           // job at the third place
 *           dispose();
 *       } else return false;
         return true;
 *   }
 * </pre>
 *
 * In above case, what the call <tt> itinerary.go("atp://second", "job2") </tt>
 * does is
 * <dt>
 * <dd> dispatches the owner aglet (specified in the argument of constructor)
 * to <tt> "atp://second" </tt>.
 * <dd> sends <tt> new Message("job2") </tt> message to the owner aglet
 * when arrived at the destination
 * </dt>
 * In this way, an aglet can specify both a destination
 * to go and a message to be handled at the destination.
 *
 * @version     1.00    $Date: 7/09/97 4:44p $
 * @author      Mitsuru Oshima
 */
// public class SimpleItinerary extends MobilityAdapter implements java.io.Serializable {
public class SimpleItinerary extends MobilityAdapter implements java.io.Externalizable {

    /**
     * Constructs a SimpleItinerary with the specified owner aglet.
     * @param aglet the owner of this itinerary.
     */
    public SimpleItinerary(Aglet aglet) {
	this.aglet = aglet;
	aglet.addMobilityListener(this);
    }

    /**
     * Goes to the destination given by name and the message processed
     * at the destination.
     * @param address the address of the destination
     * @param msg     the message being sent to the aglet at the destination
     */
    public void go(String address, Message msg) throws java.io.IOException, AgletException {
	next = msg;
	aglet.dispatch( new URL(address) );
    }

    /**
     * Goes to the destination given by name and the message processed
     * at the destination.
     * @param address the address of the destination
     * @param msg     the message being sent to the aglet at the destination
     */
    public void go(String address, String msg) throws java.io.IOException, AgletException {
	go(address, new Message(msg));
    }

    /**
     * This is not normally used by aglets programmers.
     * @param ev a mobility event
     */
    public void onArrival(MobilityEvent ev) {
	try {
	    if (next != null) {
		ev.getAgletProxy().sendAsyncMessage(next);
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    private Aglet aglet;
    private Message next;
    private Hashtable plan = new Hashtable();

    /**
     * For Manual Serialization
     */
    public SimpleItinerary() {
    }

    public void writeExternal(java.io.ObjectOutput oo) throws java.io.IOException {
	oo.writeObject(aglet);
	oo.writeObject(next);
	oo.writeObject(plan);
    }

    public void readExternal(java.io.ObjectInput in) throws java.io.IOException, ClassNotFoundException {
	aglet = (Aglet)in.readObject();
	next = (Message)in.readObject();
	plan = (Hashtable)in.readObject();
    }
}
