/*
 * @(#)Main.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package ibm.atp.daemon;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Properties;
import java.util.Enumeration;

import atci.AgentRequestHandler;

import ibm.awb.util.Resource;

/**
 * Main program for the atp daemon
 *
 * @version     1.10	3 Mar 1997
 * @author      Mitsuru Oshima
 */
final public class Main {

    static {
	try {
	    String handler_pkgs = System.getProperty("java.protocol.handler.pkgs");
	    Properties props = System.getProperties();

	    handler_pkgs = handler_pkgs + "|sun.www.net.protocol|ibm.net.protocol";
	    props.put("java.protocol.handler.pkgs", handler_pkgs);
	} catch (SecurityException ex) {
	    System.err.println("[fail to set system property 'java.protocol.handler.pkgs]");
	}
    }

    private static String agentSystem = null;

    /**
     * Main
     */
    public static void main(String args[]) throws IOException {
	if (importOptions(args) == false) {
	    usage();
	    System.exit(1);
	}
	initializeResource();

	Resource r = Resource.getResourceFor("atp");

	Daemon daemon = new Daemon();
	try {
	    daemon.setAccessLogFile( r.getString("atp.log.access") );
	    daemon.setErrorLogFile( r.getString("atp.log.error") );
	    daemon.setMessageLogFile( r.getString("atp.log.message") );
	} catch (IOException ex) {
	    ex.printStackTrace();
	}
	daemon.start(agentSystem);
    }

    /**
     * Show usage.
     */
    static public void usage() {
	String name = System.getProperty("program-name",
					 "java ibm.atp.daemon.Main");
	System.err.println("usage: " + name + " [-options] \n" +
			   "where options include: \n" +
	   "    -help                print out this message \n" +
	   "    -verbose             turn on verbose mode\n" +
	   "    -port <port>         set the port used by deamon\n" +
	   "    -viewer <class>      set the viewer class \n" +
	   "    -nosecurity          disable security manager\n" +
	   "    -manual              use the manual serialization instead of native one.\n" +
	   "    -commandline         use command line interface.\n" +
	   "    -nogui               no gui/cui.\n" +
	   "    -domain              set the domain name \n" +
	   "    -ap <agent platform> set the agentsystem");
    }

    /*
     * Initialize the system property with atp property. The properies
     * saved in the property file is automatically restored here.
     */
    private static void initializeResource() {
        /* Sets up resource
         * The defaults atp home can be set by -Datp.home=XXXX as
         * an argument to the program "java". Because the properties cannot
         * be loaded before determining the home, "atp.home" property
         * cannot be persistent. If not given, it uses "user.home" property
         * which is guaranteed. We assume that ".atp" is the default value.
         * You can give your own atp.home by -Datp.home="\foo\.XXXX".
         */
	String default_path =
		System.getProperty("user.home") + File.separator +".atp";

        String path = System.getProperty("atp.home", default_path);

	/*
	 * Load atp resources
	 */
	URL u = null;
	try {
	    u = new URL("file",
			"",
			path + File.separator + "atp.properties");
	    System.out.println("reading atp property from " + u.getFile());
	} catch (Exception ex) {
	    ex.printStackTrace();
	}

	File f = new File(u.getFile());
	Resource res = Resource.createResource("atp", u, null);
	/*
	 * Preserves all properties which are given as command line arguments.
	 */
	res.importOptionProperties("atp");
        res.setDefaultResource("atp.home", path);
	try {
	    String hostname = java.net.InetAddress.getLocalHost().getHostName();

	    if (hostname.indexOf('.')<0 &&
		res.getString("atp.domain", null) == null ) {
		System.out.println("[Warning: The hostname seems not having domain name.");
		System.out.println(" Please use -domain option to specify the domain name.]");
	    }
	} catch (java.net.UnknownHostException ex) {
	    ex.printStackTrace();
	}

	res.setDefaultResource("atp.port", res.getString("atp.defaultport","434"));

	/*
	 * Sets the names of log files
	 */
	String port = res.getString("atp.port");

	String sep = File.separator;
	String root = res.getString("atp.home","./")+sep+"logs"+sep;
	res.setDefaultResource("atp.log.access", root+"access." + port);
	res.setDefaultResource("atp.log.error",  root+"error." + port);
	res.setDefaultResource("atp.log.message", root+"message." + port);

	/*
	 * system resource
	 */
	Resource system_res = Resource.getResourceFor("system");

	/*
	 * Proxy
	 */
	system_res.setResource("http.proxyHost", 
			       res.getString("atp.http.proxyHost", ""));
	system_res.setResource("http.proxyPort",
			       res.getString("atp.http.proxyPort", ""));
	system_res.setResource("proxyHost", 
			       res.getString("atp.http.proxyHost", ""));
	system_res.setResource("proxyPort",
			       res.getString("atp.http.proxyPort", ""));

	/*
	 * aglets resource
	 */
	try {
	    u = new URL("file",
			"",
			path + File.separator + "aglets.properties");
	    System.out.println("reading aglets property from " + u.getFile());
	} catch (Exception ex) {
	    ex.printStackTrace();
	}

	Resource.createResource("aglets", u, null);
    }

    /**
     * Import option value into the system property
     */
    static private boolean importOptions(String args[]) throws IOException {
	if (args == null) {
	    return true;
	}
	Properties props = System.getProperties();

	// default viewewr of this Main program
	props.put("aglets.viewer", "ibm.aglets.tahiti.Tahiti");

	for(int i = 0; i < args.length ; i++) {
	    String option = args[i].trim();
	    if (option.equalsIgnoreCase("-defaultport")) {
		try {
		    i++;
		    int port = Integer.valueOf(args[i]).intValue();
		    if (port < 0 ) {
			throw new IllegalArgumentException("-defaultport: Illegal or missing port number.");
		    }
		    props.put("atp.defaultport", String.valueOf(port));

		} catch (NumberFormatException nfe) {
		    throw new IllegalArgumentException("-defaultport: Illegal or missing port number.");
		}
	    } else if (option.equalsIgnoreCase("-port")) {
		try {
		    i++;
		    int port = Integer.valueOf(args[i]).intValue();
		    if (port < 0 ) {
			throw new IllegalArgumentException("-port: Illegal or missing port number.");
		    }
		    props.put("atp.port", String.valueOf(port));
		} catch (NumberFormatException nfe) {
		    throw new IllegalArgumentException("-port: Illegal or missing port number.");
		}
	    } else if (option.equalsIgnoreCase("-ap")) {
		i++;
		//
		// ibm.atp.agentsystem.<agentsystem> will
		// be instantiated for this.
		//
		agentSystem = args[i];
//		props.put("atp.agentSystem",args[i]);
	    } else if (option.equalsIgnoreCase("-domain")) {
		i++;
		props.put("atp.domain", args[i]);
	    } else if (option.equalsIgnoreCase("-viewer")) {
		i++;
		props.put("aglets.viewer", args[i]);
	    } else if (option.equalsIgnoreCase("-verbose")) {
		props.put("atp.verbose", "true");
		props.put("verbose", "true");
	    } else if (option.equalsIgnoreCase("-nosecurity")) {
		props.put("aglets.secure", "false");
	    } else if (option.equalsIgnoreCase("-manual")) {
		props.put("aglets.manualserialization", "true");
	    } else if (option.equalsIgnoreCase("-nogui")) {
		props.put("aglets.viewer","");
	    } else if (option.equalsIgnoreCase("-commandline")) {
		props.put("aglets.viewer",
			  "ibm.aglets.tahiti.CommandLine");
	    } else if (option.equalsIgnoreCase("-help")) {
		return false;

	    //
	    // Hidden Options
	    //
	    } else if (option.equalsIgnoreCase("-startup")) {
		i++;
		props.put("aglets.startup", "true");
		props.put("aglets.startupAglets", args[i]);
	    } else if (option.equalsIgnoreCase("-enablebox")) {
		props.put("aglets.enableBox", "true");
	    } else if (option.equalsIgnoreCase("-newsecurity")) {
		props.put("aglets.security.policy", "idbase");

	    //
	    // for debugging
	    //
	    } else if (option.equalsIgnoreCase("-err")) {
		i++;
		System.err =
			new java.io.PrintStream(new java.io.FileOutputStream(args[i]));
	    } else if (option.equalsIgnoreCase("-out")) {
		i++;
		System.out =
			new java.io.PrintStream(new java.io.FileOutputStream(args[i]));
	    } else {
		System.out.println("unknown option :[" + option + "]");
		return false;
	    }
	}
	return true;
    }
}
